using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using SimpleJSON;
using System.Threading;
using MVR;
using MeshVR;
using System.Linq;
using Leap.Unity;


namespace Stopper {
	public class ClothingPluginManager : MVRScript {

		DAZDynamic dazDynamic;
		DAZClothingItem dazClothingItem;
		MVRPluginManager pluginManager;

		public override void Init() {
			
			WalkAndGetParentItems();
			
			if(containingAtom == null) throw new Exception("Unable to determine containing atom.");
			if(pluginManager == null) throw new Exception("Unable to find plugin manager.");
			if(dazClothingItem == null) throw new Exception("Unable to find clothing item.");

			this.overrideId = $"{dazDynamic.uid}:{this.storeId}";

			pluginManager.containingAtom = containingAtom;

			// Copy over all of the prefabs
			MVRPluginManager containingAtomPluginManager = containingAtom.GetStorableByID("PluginManager") as MVRPluginManager;
			if(containingAtomPluginManager != null){
				pluginManager.pluginPanelPrefab = containingAtomPluginManager.pluginPanelPrefab;
				pluginManager.scriptControllerPanelPrefab = containingAtomPluginManager.scriptControllerPanelPrefab;
				pluginManager.scriptUIPrefab = containingAtomPluginManager.scriptUIPrefab;
				pluginManager.configurableSliderPrefab = containingAtomPluginManager.configurableSliderPrefab;
				pluginManager.configurableTogglePrefab = containingAtomPluginManager.configurableTogglePrefab;
				pluginManager.configurableColorPickerPrefab = containingAtomPluginManager.configurableColorPickerPrefab;
				pluginManager.configurableButtonPrefab = containingAtomPluginManager.configurableButtonPrefab;
				pluginManager.configurablePopupPrefab = containingAtomPluginManager.configurablePopupPrefab;
				pluginManager.configurableScrollablePopupPrefab = containingAtomPluginManager.configurableScrollablePopupPrefab;
				pluginManager.configurableFilterablePopupPrefab = containingAtomPluginManager.configurableFilterablePopupPrefab;
				pluginManager.configurableTextFieldPrefab = containingAtomPluginManager.configurableTextFieldPrefab;
				pluginManager.configurableSpacerPrefab = containingAtomPluginManager.configurableSpacerPrefab;
			} else {
				throw new Exception("Unable to find plugin manager for main atom.");
			}
		}

		/*
			There might be a more elegant way to do this but this works for now.
		*/
		private void WalkAndGetParentItems(){
			GameObject parent = this.gameObject;
			while(this.containingAtom == null || parent != this.containingAtom.gameObject){
				if(parent.transform.parent == null){
					return;
				}
				if(pluginManager == null){
					pluginManager = parent.GetComponent<MVRPluginManager>();
				}
				if(dazDynamic == null){
					dazDynamic = parent.GetComponent<DAZDynamic>();
				}
				if(dazClothingItem == null){
					dazClothingItem = parent.GetComponent<DAZClothingItem>();
				}
				this.containingAtom = parent.GetComponent<Atom>();
				parent = parent.transform.parent.gameObject;
			}
		}
		
		private void Start(){
			// Add the destructor to unload mods if we do a force reload
			var destruct = dazDynamic.binaryStorableBucket.gameObject.AddComponent<Destructor>();
			destruct.pluginManager = pluginManager;
			destruct.containingAtom = containingAtom;
			pluginManager.containingAtom = containingAtom;

			StartCoroutine("AddPluginTabToUI");
		}

		public IEnumerator AddPluginTabToUI(){
			MVRPluginManager containingAtomPluginManager = containingAtom.GetStorableByID("PluginManager") as MVRPluginManager;
			if(containingAtomPluginManager == null) throw new Exception("Cannot find plugin manager for parent atom.");
			
			// Wait for the Person's plugin manager UI to be loaded.
			// This should happen when the Person's UI is first opened.
            Transform pluginManagerUITransform = null;
			while(pluginManagerUITransform == null) {
				yield return null;
				pluginManagerUITransform = containingAtomPluginManager.UITransform;
			}

			// Wait for the clothing item's customization UI to be created
			// This should happen when the user clicks 'Customize' on the clothing
			Transform customizationUI = dazClothingItem.customizationUI;
			while(customizationUI == null) {
				yield return null;
				customizationUI = dazClothingItem.customizationUI;
			}

			// Wait for the customization UI to be active
			while(!customizationUI.gameObject.activeInHierarchy) {
				yield return null;
			}
			
			// This should have been initialized with the PluginManager's IO but I've seen it null here. 
			// Race conditions? Me being dumb? Who knows. Adding wait loop.
			MVRPluginManagerUI containingAtomPluginManagerUI = pluginManagerUITransform.GetComponentInChildren<MVRPluginManagerUI>(true);
			while(containingAtomPluginManagerUI == null) {
				yield return new WaitForSeconds(0.1f);
				containingAtomPluginManagerUI = pluginManagerUITransform.GetComponentInChildren<MVRPluginManagerUI>(true);
			}

			UIConnectorMaster uiConnectorMaster = customizationUI.GetComponent<UIConnectorMaster>();
			TabbedUIBuilder tabbedUIBuilder = uiConnectorMaster.GetComponentInChildren<TabbedUIBuilder>();

			foreach(var child in tabbedUIBuilder.selector.toggleContainer.GetChildren()) {
				if(child.name == "Plugins") {
					yield break;
				}
			}
			
			Transform pluginTabPrefab = UnityEngine.Object.Instantiate<Transform>(containingAtomPluginManagerUI.transform,tabbedUIBuilder.selector.transform);

			// Remove any plugin UIs we inadvertently copied over
			foreach(var script in pluginTabPrefab.GetComponentsInChildren<MVRScriptUI>(true)) {
				Destroy(script.gameObject);
			}

			foreach(var script in pluginTabPrefab.GetComponentsInChildren<MVRPluginUI>(true)) {
				Destroy(script.gameObject);
			}

			
			TabbedUIBuilder.Tab pluginsTab = new TabbedUIBuilder.Tab{name = "Plugins", prefab = pluginTabPrefab, color = Color.blue};
			tabbedUIBuilder.AddTab(pluginsTab);

			pluginManager.UITransform = pluginTabPrefab.transform;
			pluginManager.InitUI();
			yield break;
		}

		public override void RestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, JSONArray presetAtoms = null, bool setMissingToDefault = true){
            base.RestoreFromJSON(jc,restorePhysical,restoreAppearance,presetAtoms,false);
        }

		/*
			A forced reload of the parent clothing will only clear specific
			types of components. MaterialOptions are one of them, and should 
			otherwise be inert. 

			Using this to detect that the clothing is reloading and clearing
			the plugins. 

			Not doing this causes the plugin managers and plugins to accumulate.
		*/
		public class Destructor : MaterialOptions {
			public MVRPluginManager pluginManager = null;
			public void OnDestroy(){
				if(pluginManager != null){
					pluginManager.containingAtom = containingAtom;
					pluginManager.RemoveAllPlugins();
					Destroy(pluginManager);
				}
			}
		}
    }
}
